unit CanCollect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  ECanError = class (Exception);

  TMyItem = class (TCollectionItem)
  private
    FCode: Integer;
    FText: string;
    procedure SetCode(const Value: Integer);
    procedure SetText(const Value: string);
  published
    property Text: string read FText write SetText;
    property Code: Integer read FCode write SetCode;
  end;

  TMyCollection = class (TCollection)
  private
    fComp: TComponent;
    fCollString: string;
  public
    constructor Create (CollOwner: TComponent);
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  end;

  TCanTest = class(TComponent)
  private
    ftesto: string;
    FKey: Char;
    fOnChange: TNotifyEvent;
    FCount: Integer;
    fcoll: TMyCollection;

    procedure ReadEt (Reader: TReader);
    procedure WriteEt (Writer: TWriter);
    procedure SetKey(const Value: Char);
    function GetKey: Char;
    procedure SetCount(const Value: Integer);
    function GetOutString: string;
    procedure SetOutString(const Value: string);
    procedure SetMoreData(const Value: TMyCollection);
    procedure SetCollString(const Value: string);
    function GetCollString: string;
  protected
  public
    constructor Create (aOwner: TComponent); override;
    destructor Destroy; override;
    procedure DefineProperties (Filer: TFiler); override;
  published
    property Key: Char
      read GetKey write SetKey default 'a';
    property Count: Integer
      read FCount write SetCount default 1;
    property MoreData: TMyCollection
      read FColl write SetMoreData;
    property OutString: string
      read GetOutString write SetOutString
      stored False;
    property OnChange: TNotifyEvent
      read fOnChange write fOnChange;
    property CollString: string
      read GetCollString write SetCollString stored False;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TCanTest]);
end;

{ TCanTest }

constructor TCanTest.Create(aOwner: TComponent);
begin
  inherited;
  fKey := 'a';
  fCount := 1;
  ftesto := '';
  fColl := TMyCollection.Create (self);
end;

procedure TCanTest.DefineProperties (Filer: TFiler);
begin
  inherited DefineProperties (Filer);
  Filer.DefineProperty ('ErrorText', ReadEt, WriteEt,
    fTesto <> '');
end;

destructor TCanTest.Destroy;
begin
  inherited;
  fColl.Free;
end;

function TCanTest.GetCollString: string;
begin
  Result := fColl.fCollString;
end;

function TCanTest.GetKey: Char;
begin
  if Ord (fKey) < 32 then
    Result := ' '
  else
    Result := fKey;
end;

function TCanTest.GetOutString: string;
begin
  // SetLength (Result, fCount + 1);
  Result := StringOfChar (fKey, fCount);
end;

procedure TCanTest.ReadEt(Reader: TReader);
begin
  fTesto := Reader.ReadString;
end;

procedure TCanTest.SetCollString(const Value: string);
begin
  // do nothing (or provide a conversion method)
end;

procedure TCanTest.SetCount(const Value: Integer);
begin
  FCount := Value;
end;

procedure TCanTest.SetKey(const Value: Char);
begin
  if fKey <> Value then
  begin
    if Value = #13 then
      raise ECanError.Create ('Invalid value 13');
    fKey := Value;
    // side effect
    if Assigned (fOnChange) then
      fOnChange (self);
    // ShowMessage (fKey);
  end;
end;

procedure TCanTest.SetMoreData(const Value: TMyCollection);
begin
  FColl.Assign (Value);
end;

procedure TCanTest.SetOutString(const Value: string);
var
  n: integer;
begin
  fTesto := Value;
  ((fColl.Add) as TMyItem).Text := fTesto;

  if Value <> OutString then
  begin
    for n := 2 to Length (Value) do
      if Value [n] <> Value [1] then
        raise ECanError.Create ('Not same character');

    // assign values
    fKey := Value [1];
    fCount := Length (Value);
  end;
end;

procedure TCanTest.WriteEt(Writer: TWriter);
begin
  Writer.WriteString (fTesto);
end;

{ TMyItem }

procedure TMyItem.SetCode(const Value: Integer);
begin
  FCode := Value;
  Changed (False);
end;

procedure TMyItem.SetText(const Value: string);
begin
  FText := Value;
  Changed (False);
end;

{ TMyCollection }

constructor TMyCollection.Create (CollOwner: TComponent);
begin
  inherited Create (TMyItem);
  fComp := CollOwner;
end;

function TMyCollection.GetOwner: TPersistent;
begin
  Result := fComp;
end;

procedure TMyCollection.Update(Item: TCollectionItem);
var
  str: string;
  i: integer;
begin
  inherited;
  // udpate all in any case
  str := '';
  for i := 0 to Count - 1 do
  begin
    str := str + (Items [i] as TMyItem).Text;
    if i < Count - 1 then
      str := str + '-';
  end;
  fCollString := str;
end;

end.
