unit CanContainer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls;

type
  TCanContainer = class(TWinControl)
  private
    FHasLabel: Boolean;
    fLabel: TLabel;
    procedure SetHasLabel(const Value: Boolean);
    procedure SetLabelCaption(const Value: string);
    function GetLabelCaption: string;
    { Private declarations }
  protected
  public
    constructor Create (AOwner: TComponent); override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Loaded; override;
  published
    property HasLabel: Boolean
      read FHasLabel write SetHasLabel
      default False;
    property LabelCaption: string
      read GetLabelCaption write SetLabelCaption stored False;
    property Width default 100;
    property Height default 100;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TCanContainer]);
end;

{ TCanContainer }

constructor TCanContainer.Create(AOwner: TComponent);
begin
  inherited;
  Width := 100;
  Height := 100;
  ControlStyle := ControlStyle + [csAcceptsControls, csFramed, csFixedWidth];
end;

procedure TCanContainer.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
  inherited;
  if Assigned (fLabel) then
    Proc (fLabel);
end;

function TCanContainer.GetLabelCaption: string;
begin
  if Assigned (fLabel) then
    Result := fLabel.Caption
  else
    Result := '';
end;

procedure TCanContainer.Loaded;
begin
  inherited;
  if (HasLabel = True) then
  begin
    fLabel := GetParentForm (self).FindComponent (
      'CanContainerInternalLabel') as TLabel;
    if Assigned (fLabel) then
    begin
      GetParentForm (self).RemoveComponent (fLabel);
      InsertComponent (fLabel);
    end;
  end;
end;

procedure TCanContainer.SetHasLabel(const Value: Boolean);
begin
  if fHasLabel <> Value then
  begin
    FHasLabel := Value;
    if not (csLoading in ComponentState) then
      if fHasLabel then
      begin
        fLabel := TLabel.Create (self);
        fLabel.Parent := self;
        fLabel.Align := alTop;
        fLabel.Caption := 'My Label';
        fLabel.Name := 'CanContainerInternalLabel'; // unique
      end
      else
      begin
        fLabel.Free;
        fLabel := nil;
      end;
  end;
end;

procedure TCanContainer.SetLabelCaption(const Value: string);
begin
  if Assigned (fLabel) then
    fLabel.Caption := Value;
end;

initialization
  RegisterClasses ([TLabel]);

end.
