unit CanLed;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
   Forms, Dialogs, ExtCtrls, stdctrls;

type
  TCanLed = class(TGraphicControl)
  private
    FColor: TColor;
    fPen: TPen;
    fTimer: TTimer;
    fLabel: TLabel;
    procedure SetColor(const Value: TColor);
    procedure SetPen(const Value: TPen);
    procedure SetInterval(const Value: Integer);
    function GetInterval: Integer;
    procedure SetLabel(const Value: TLabel);
  protected
    procedure PenChange (Sender: TObject);
    procedure TimerChange (Sender: TObject);
    procedure UpdateLabel;
  public
    constructor Create (aOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure Notification (AComponent: TComponent;
      Operation: TOperation); override;
  published
    property Align;
    property Visible;
    property OnClick;
    property OnMouseMove;
    property Enabled;
    property ShowHint;
    property Width default 50;
    property Height default 50;
    property OutLabel: TLabel
      read fLabel write SetLabel;
    property Color: TColor
      read FColor write SetColor default clGreen;
    property Pen: TPen read FPen write SetPen;
    property Interval: Integer
      read GetInterval write SetInterval;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TCanLed]);
end;

{ TCanLed }

constructor TCanLed.Create(aOwner: TComponent);
begin
  inherited;
  Width := 50;
  Height := 50;
  fColor := clGreen;
  fPen := TPen.Create;
  fPen.OnChange := PenChange;
  fTimer := TTimer.Create (self);
  fTimer.Enabled := True;
  if not (csDesigning in ComponentState) then
    fTimer.OnTimer := TimerChange;
end;

destructor TCanLed.Destroy;
begin
  inherited;
  fPen.Free;
end;

function TCanLed.GetInterval: Integer;
begin
  Result := fTimer.Interval;
end;

procedure TCanLed.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (AComponent = fLabel) and (Operation = opRemove) then
    fLabel := nil;
end;

procedure TCanLed.Paint;
var
  str: string;
  rec: TRect;
begin
  inherited;
  Canvas.Brush.Color := fColor;
  Canvas.Pen.Assign (fPen);
  Canvas.Ellipse (0, 0, Width, Height);
  str := TimeToStr (Now);
  rec := Rect (0, 0, Width, Height);
  DrawText (Canvas.Handle, pchar (str), Length (str), rec,
    dt_center or dt_vcenter or dt_singleline);
end;

procedure TCanLed.PenChange(Sender: TObject);
begin
  Invalidate;
end;

procedure TCanLed.SetColor(const Value: TColor);
begin
  if fColor <> Value then
  begin
    FColor := Value;
    Invalidate;
    UpdateLabel;
  end;
end;

procedure TCanLed.SetInterval(const Value: Integer);
begin
  fTimer.Interval := Value;
end;

procedure TCanLed.SetLabel(const Value: TLabel);
begin
  if FLabel <> Value then
  begin
    fLabel := Value;
    if fLabel <> nil then
    begin
      UpdateLabel;
      fLabel.FreeNotification (self);
    end;
  end;
end;

procedure TCanLed.SetPen(const Value: TPen);
begin
  fPen.Assign (Value);
  Invalidate; // not sure if needed 
end;

procedure TCanLed.TimerChange(Sender: TObject);
begin
  // optionally change also the color
  // Color := RGB (Random (255), Random (255), Random (255));
  Invalidate;
end;

procedure TCanLed.UpdateLabel;
begin
  if Assigned (fLabel) then
    fLabel.Caption := ColorToString (fColor);
end;

end.
