unit CantDbTrack;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, DB, DBCtrls;

type
  TCanDbTrack = class(TTrackBar)
  private
    FDataLink: TFieldDataLink;
    function GetDataField: string;
    procedure SetDataField (Value: string);
    function GetDataSource: TDataSource;
    procedure SetDataSource (Value: TDataSource);
    function GetField: TField;
    procedure CNHScroll(var Message: TWMHScroll); message CN_HSCROLL;
    procedure CNVScroll(var Message: TWMVScroll); message CN_VSCROLL;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    // data link event handlers
    procedure DataChange (Sender: TObject);
    procedure UpdateData (Sender: TObject);
    procedure ActiveChange (Sender: TObject);
//    procedure WndProc (var Msg: TMessage); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
  published
    property DataField: string
      read GetDataField write SetDataField;
    property DataSource: TDataSource
      read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation

constructor TCanDbTrack.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;
  Enabled := False;
end;

destructor TCanDbTrack.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

function TCanDbTrack.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TCanDbTrack.SetDataField (Value: string);
begin
  try
    FDataLink.FieldName := Value;
  finally
    Enabled := FDataLink.Active and
      (FDataLink.Field <> nil) and
      not FDataLink.Field.ReadOnly;
  end;
end;

function TCanDbTrack.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCanDbTrack.SetDataSource (Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  Enabled := FDataLink.Active and
    (FDataLink.Field <> nil) and
    not FDataLink.Field.ReadOnly;
end;

function TCanDbTrack.GetField: TField;
begin
  Result := FDataLink.Field;
end;

// data link event handler
procedure TCanDbTrack.DataChange (Sender: TObject);
begin
  if (FDataLink.Field <> nil) and
      (FDataLink.Field is TNumericField) then
    Position := FDataLink.Field.AsInteger
  else
    Position := Min;
end;

procedure TCanDbTrack.ActiveChange (Sender: TObject);
begin
  Enabled := FDataLink.Active and
    (FDataLink.Field <> nil) and
    not FDataLink.Field.ReadOnly;
end;

// update
procedure TCanDbTrack.CNHScroll(var Message: TWMHScroll);
var
  OldPos: Integer;
begin
  // edit mode
  if FDataLink.Edit then
  begin
    // update data
    inherited;
    // let the system know
    FDataLink.Modified;
  end
  else
  begin
    OldPos := FDataLink.Field.AsInteger;
    inherited;
    // reset the trackbar position
    Position := OldPos;
  end;
end;

procedure TCanDbTrack.CNVScroll(var Message: TWMVScroll);
var
  OldPos: Integer;
begin
  // edit mode
  if FDataLink.Edit then
  begin
    // update data
    inherited;
    // let the system know
    FDataLink.Modified;
  end
  else
  begin
    OldPos := FDataLink.Field.AsInteger;
    inherited;
    // reset the trackbar position
    Position := OldPos;
  end;
end;

procedure TCanDbTrack.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TCanDbTrack.UpdateData (Sender: TObject);
begin
  if (FDataLink.Field <> nil) and
      (FDataLink.Field is TNumericField) then
    FDataLink.Field.AsInteger := Position;
end;

procedure Register;
begin
  RegisterComponents('Cantools', [TCanDbTrack]);
end;

{procedure TCanDbTrack.WndProc(var Msg: TMessage);
begin
  if (Msg.Msg = WM_LBUTTONDOWN) or (Msg.Msg = WM_SYSCHAR) or
      (Msg.Msg = WM_LBUTTONDBLCLK) then
    if not FDataLink.Edit then
      inherited;
  inherited;
end;}

end.
